package com.bomostory.sceneeditmodule.screen.view

import android.content.Context
import android.graphics.Paint
import android.graphics.drawable.Drawable
import android.support.constraint.ConstraintLayout
import android.util.AttributeSet
import android.util.DisplayMetrics
import android.view.ViewGroup
import android.view.WindowManager
import android.widget.RelativeLayout
import com.bomostory.sceneeditmodule.basicdata.Actor
import com.bomostory.sceneeditmodule.basicdata.Scene
import kotlin.math.pow

class MovieView : ConstraintLayout {

    private var backgroundView = LayerView(context)

    var scene: Scene? = null
        set(value) {
            field = value
            value?.let {
                removeAllViews()

                val screenWidth = it.sceneWidth
                val screenHeight = it.sceneWidth / 2
                val widthScaleFactor: Float = layoutParams.width / screenWidth.toFloat()
                val heightScaleFactor: Float = layoutParams.height / screenHeight.toFloat()

                backgroundView.apply {
                    val width = (it.sceneWidth * 33 / 32f * widthScaleFactor).toInt()
                    val height = (it.sceneWidth / 2 * heightScaleFactor).toInt()
                    var lParams = RelativeLayout.LayoutParams(width, height)
                    layoutParams = lParams
                    background = Drawable.createFromPath(it.backgroundPath)
                    x = -width / 64f
                }
                addView(backgroundView)

                val CONTROLLER_RADIUS = 25
                for (layer in it.layers) {
                    val layerView = LayerView(context)

                    val actorsFix = ArrayList<Actor>()
                    for (actor in layer.actors) {
                        val actorFix = Actor().apply {
                            resourcePath = actor.resourcePath
                            if (!actor.isDialogue) {
                                positionX = ((actor.positionX + CONTROLLER_RADIUS * 2) * widthScaleFactor).toInt()
                                positionY = ((actor.positionY + CONTROLLER_RADIUS * 2) * heightScaleFactor).toInt()
                                sideLength = ((actor.sideLength - CONTROLLER_RADIUS * 4) * widthScaleFactor).toInt()
                                sideHeight = ((actor.sideHeight - CONTROLLER_RADIUS * 4) * heightScaleFactor).toInt()
                            }
                            else {
                                positionX = actor.positionX
                                positionY = actor.positionY
                                sideLength = actor.sideLength
                                sideHeight = actor.sideHeight
                            }
                            positionZ = actor.positionZ
                            isSelect = actor.isSelect
                            parentLayerIndex = actor.parentLayerIndex
                            isMovable = actor.isMovable
                            text = actor.text
                            textColor = actor.textColor
                            textAlign = actor.textAlign
                            isDialogue = actor.isDialogue
                            isMirror = actor.isMirror
                            opacity = actor.opacity
                            dialogColor = actor.dialogColor
                            dialogType = actor.dialogType
                        }
                        actorsFix.add(actorFix)
                    }

                    layerView.apply {
                        id = layer.id
                        actors = actorsFix
                    }
                    addView(layerView)
                }
            }
        }

    var x: Int = 0
        set(value) {
            field = value
            scene?.let {

                val screenWidth = it.sceneWidth
                val widthScaleFactor: Float = layoutParams.width / screenWidth.toFloat()

                backgroundView.x = value * widthScaleFactor / 32f - backgroundView.width / 64f

                for (layer in it.layers) {
                    val layerView = findViewById<LayerView>(layer.id)
                    layerView.x = value * widthScaleFactor / (2f.pow(it.layers.indexOf(layer)))
                }
            }
        }

    constructor(context: Context) : super(context)

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)

    constructor(context: Context, attrs: AttributeSet, defStyle: Int) : super(context, attrs, defStyle)

}